// $Header: /n/basmati/da/bedk/speech/src/drspeech_featools/RCS/OnlfeatureInput.cc,v 1.1 1997/07/29 17:54:37 bedk Exp $
//
// $Log: OnlfeatureInput.cc,v $
// Revision 1.1  1997/07/29 17:54:37  bedk
// Initial revision
//
//
// Object for reading online feature streams
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <assert.h>

#include "OnlfeatureInput.h"

static void swab_fbuf(float *buf, int n) {
  // Swap bytes in a buffer of floats, if needed to make them bigendian
#ifndef WORDS_BIGENDIAN
  unsigned int *ptr = (unsigned int *)buf;
  unsigned int d;
  int i;

  assert(sizeof(unsigned int) == sizeof(float));
  for (i=0; i<n; ++i) {
    d = *ptr;
    *ptr++ = ((d & 0x000000FF) << 24) | ((d & 0x0000FF00) << 8) | ((d & 0x00FF0000) >> 8) | (0x000000FF & ((d & 0xFF000000) >> 24));
  }
#endif /* WORDS_BIGENDIAN */
}

OnlfeatureInput::OnlfeatureInput(const int a_IOblocking,
                                 const int a_nchan,
                                 const char* const fileName)
  : IOBlocking(a_IOblocking),
    nchan(a_nchan),
    lookahead(0),
    lookahead_full(false)
{
  if (fileName == NULL)
    {
      fprintf(stderr,
              "NULL file name, exiting.\n");
      exit(1);
    }
  else if (!strcmp(fileName,"-"))
    {
      file = stdin;
      have_stdin = true;
    }
  else
    {
      have_stdin = false;
      if ((file = fopen(fileName,"r")) == NULL)
        {
          fprintf(stderr,
                  "Can't open file (%s), exiting.\n",
                  fileName);
          exit(1);    
        }
    }

  buf = new float[nchan];
  result = new Matrix(nchan, IOBlocking);
}

OnlfeatureInput::~OnlfeatureInput()
{
  if (!have_stdin) { fclose(file); }
  delete(buf);
  delete(result);
}

const CMatrix
OnlfeatureInput::read(bool& EOUtt, bool& EOProc, bool& EOFile)
{
  unsigned char flag;
  size_t n;
  size_t nread = 0;
  float* p = result->data;

  if (EOUtt)
    { 
      EOProc = true;
      return CMatrix(0, *result);
    }

  while (nread < (size_t) IOBlocking)
    {
      if (lookahead_full)
        {
          lookahead_full = false;
          n = 1;
          flag = lookahead;
        }
      else
        {
          n = fread(&flag, sizeof(char), 1, file);
        }
      
      if (n == 0)
        {
          if (feof(file))
            {
              EOUtt = true;
              EOProc = true;
              EOFile = true;
              break;
            }
          else
            {
              fprintf(stderr, "Error reading on-line input.\n");
              exit(1);
            }
        }

      if (fread(buf, sizeof(float), nchan, file) != nchan)
        {
          fprintf(stderr, "Error reading on-line input.\n");
          exit(1);
       }
      swab_fbuf(buf, nchan);

      float* q = p++;
      for (float* bp = buf;
           bp != buf + nchan;
           bp++)
        {
          *q = *bp;
          q += result->s;
        }
      nread++;

      if (flag == 0x80)
        {
          EOUtt = true;
          EOProc = true;
          n = fread(&lookahead, sizeof(char), 1, file);
          if (n == 0)
            {
              if (feof(file))
                {
                  EOFile = true;
                }
              else
                {
                  fprintf(stderr, "Error reading on-line input.\n");
                  exit(1);
                }
            }
          else
            {
              lookahead_full = true;
            }
          break;
        }
      else if (flag != 0)
        {
          fprintf(stderr, "Error reading on-line input.\n");
          exit(1);
       }
    }

  return CMatrix(nread, *result);
}
