// $Header: /n/yam/da/dpwe/tmp/bedk_frontend/RCS/OnlfeatureOutput.cc,v 1.1 1997/07/08 22:13:12 bedk Exp dpwe $
//
// $Log: OnlfeatureOutput.cc,v $
// Revision 1.1  1997/07/08 22:13:12  bedk
// Initial revision
//
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "OnlfeatureOutput.h"

OnlfeatureOutput::OnlfeatureOutput(const int a_nchan,
                                   const char* const fileName)
  : nchan(a_nchan),
    buffer(new float[nchan]),
    buffer_full(false)
{
  if (fileName == NULL)
    {
      fprintf(stderr,
              "NULL file name, exiting.\n");
      exit(1);
    }
  else if (strcmp(fileName, "-") == 0)
    {
      file = stdout;
      have_stdout = true;
    }
  else
    {
      have_stdout = false;
      if ((file = fopen(fileName,"w")) == NULL)
        {
          fprintf(stderr,
                  "Unable to open %s for output.\n",
                  fileName);
          exit(1);
        }
    }
}

static void swab_fbuf(float *buf, int n) {
  // Swap bytes in a buffer of floats, if needed to make them bigendian
#ifndef WORDS_BIGENDIAN
  unsigned int *ptr = (unsigned int *)buf;
  unsigned int d;
  int i;

  assert(sizeof(unsigned int) == sizeof(float));
  for (i=0; i<n; ++i) {
    d = *ptr;
    *ptr++ = ((d & 0x000000FF) << 24) | ((d & 0x0000FF00) << 8) | ((d & 0x00FF0000) >> 8) | (0x000000FF & ((d & 0xFF000000) >> 24));
  }
#endif /* WORDS_BIGENDIAN */
}

OnlfeatureOutput::~OnlfeatureOutput()
{
  if (buffer_full)
    {
      const unsigned char EOS = 0x80;
      if (fwrite(&EOS, sizeof(unsigned char), 1, file) != 1)
        {
          fprintf(stderr, "Error writing online feature stream.\n");
          exit(1);
        }
      swab_fbuf(buffer, nchan);
      if (fwrite(buffer, sizeof(float), nchan, file) != nchan)
        {
          fprintf(stderr, "Error writing online feature stream.\n");
          exit(1);
        }
      swab_fbuf(buffer, nchan);
    }
  if (!have_stdout) { fclose(file); }
  delete buffer;
}

void
OnlfeatureOutput::write(const CMatrix m,
                        bool& lastTime)
{
  assert(m.m == nchan);

  if (m.n != 0)
    {
      unsigned char flag = 0;

      // If there is data in the output buffer, write it out
      //
      if (buffer_full)
        {
          if (fwrite(&flag, sizeof(unsigned char), 1, file) != 1)
            {
              fprintf(stderr, "Error writing online feature stream.\n");
              exit(1);
            }
	  swab_fbuf(buffer, nchan);
          if (fwrite(buffer, sizeof(float), nchan, file) != nchan)
            {
              fprintf(stderr, "Error writing online feature stream.\n");
              exit(1);
            }
	  swab_fbuf(buffer, nchan);
        }

      // write all but one column of the input matrix out
      //
      const float* col_start = m.data;
      const float* col_end = m.data + m.m * m.s;
      const float*  col_stop = m.data + m.n - 1;
      while (col_start != col_stop)
        {
          if (fwrite(&flag, sizeof(unsigned char), 1, file) != 1)
            {
              fprintf(stderr, "Error writing online feature stream.\n");
              exit(1);
            }

          float* q = buffer;
          for (const float* p = col_start;
               p != col_end;
               p += m.s)
            {
              *q++ = *p;
            }

	  swab_fbuf(buffer, nchan);
          if (fwrite(buffer, sizeof(float), nchan, file) != nchan)
            {
              fprintf(stderr, "Error writing online feature stream.\n");
              exit(1);
            }
	  swab_fbuf(buffer, nchan);
          
          col_start++;
          col_end++;
        }
      
      // put the last column of the input matrix into the output
      // buffer.  If we are at the end of an utterance, write it out;
      // otherwise, keep it in the buffer.
      //
      float* q = buffer;
      for (const float* p = col_start;
           p != col_end;
           p += m.s)
        {
          *q++ = *p;
        }
      
      if (lastTime)
        {
          flag = 0x80;

          if (fwrite(&flag, sizeof(unsigned char), 1, file) != 1)
            {
              fprintf(stderr, "Error writing online feature stream.\n");
              exit(1);
            }

	  swab_fbuf(buffer, nchan);
          if (fwrite(buffer, sizeof(float), nchan, file) != nchan)
            {
              fprintf(stderr, "Error writing online feature stream.\n");
              exit(1);
            }
	  swab_fbuf(buffer, nchan);
          
          buffer_full = false;
        }
      else
        {
          buffer_full = true;
        }
    }
  else if (lastTime && buffer_full)
    {
      unsigned char flag = 0x80;

      if (fwrite(&flag, sizeof(unsigned char), 1, file) != 1)
        {
          fprintf(stderr, "Error writing online feature stream.\n");
          exit(1);
        }
      swab_fbuf(buffer, nchan);
      if (fwrite(buffer, sizeof(float), nchan, file) != nchan)
        {
          fprintf(stderr, "Error writing online feature stream.\n");
          exit(1);
        }
      swab_fbuf(buffer, nchan);
          
      buffer_full = false;
    }
}
