// $Id: PowspecFiltbank.h,v 1.3 1999/01/01 01:11:57 dpwe Exp dpwe $
// 
// $Log: PowspecFiltbank.h,v $
// Revision 1.3  1999/01/01 01:11:57  dpwe
// *** empty log message ***
//
// Revision 1.2  1998/06/08 18:13:41  bedk
// Added Bark-scale filters
//
// Revision 1.1  1998/05/28 21:12:40  bedk
// Initial revision
//
// 
// Simulates a filterbank by generating FFT power spectra and doing a
// weighted accumulation of the power spectral points.
// 

#ifndef PowspecFiltbank_h
#define PowspecFiltbank_h

#include "Vector.h"
#include "Matrix.h"
#include "Blocker.h"

extern "C"
{
#include "fft.h"
}

class PowspecFiltbank : public Blocker
{
public:
  enum FilterShape {
    TRAPEZOIDAL = 0,
    TRIANGULAR
  };

  enum FilterScale {
    CONST_Q = 0,
    BARK
  };

  PowspecFiltbank(const size_t IOBlocking,
                  const size_t nfft,
                  const size_t winStep,
                  const Vector& window,
                  const float bw,
                  const float filterStep,
                  const float tw,
                  const float sf,
                  const float loCutoff,
                  const FilterShape filterShape,
                  const FilterScale filterScale,
                  const bool dither = false, 
		  const float a_nyq_frac = 1.0);

  ~PowspecFiltbank();

  static size_t computeNumChannels(const FilterScale a_filterScale,
                                   const float a_bw,
                                   const float a_filterStep,
                                   const float a_sf,
                                   const float a_loCutoff, 
				   const float a_nyq_frac = 1.0)
  {
    double tn = 0.0;
    size_t n = 0;

    switch (a_filterScale)
      {
      case CONST_Q:
        tn = 1.0 + ceil((-1.0 - a_bw +
                         (log(a_sf*a_nyq_frac) - log(a_loCutoff)) / log(2.0)) /
                        a_filterStep);
        break;
            
      case BARK:
        tn = 1.0 + ceil((6.0 * (asinh(a_sf*a_nyq_frac/1200.0) - 
                                asinh(a_loCutoff/600.0)) -
                         a_bw) /
                        a_filterStep);
        break;
      }

    if (tn < 1.0)
      {
        n = 1;
      }
    else
      {
        n = (size_t) tn;
      }
    
    return n;
  };

  const CMatrix processBlock(const CVector in,
                             bool& lastTime);
                   
private:
  const size_t IOBlocking;
  const size_t nfft;
  const size_t winStep;
  const size_t indexMax;
  const Vector window;
  size_t bufLen;
  size_t pointsToNextWindow;
  float* inBuffer;
  float* FFTBuffer;
  const float* const win_endp;
  const float* const fft_endp;
  float* reorder;
  float* basis;
  size_t* index;
  const float bw;
  const float filterStep;
  const float tw;
  const float sf;
  const float nyq_frac;
  const float loCutoff;
  const FilterShape filterShape;
  const FilterScale filterScale;
  const bool dither;
  const size_t numChannels;
  float* outBuffer;
  size_t* filt_npts;
  size_t* filt_start;
  float* filt_weights;

  float dotproduct(const size_t n,
                   const float* avec,
                   const float* bvec);
};

#endif // PowspecFiltbank_h
