/*
  fft.h --- interface spec for FFT routines
*/

#ifndef fft_h_INCLUDED
#define fft_h_INCLUDED

#include <stddef.h>

/* Radix-2, decimation-in-time FFT.  Works only on vectors with
   power-of-two lengths.  Operates in-place on vector x.  The input
   is in bit-reversed order.

   w is the basis vector (aka "twiddle factors").
   Both x and w are vectors of complex numbers, arranged as
   re[0] im[0] re[1] im[1] ...

   This FFT routine may be used as an inverse FFT by providing a
   different basis vector (see initInvBasis), and dividing x by nx on
   completion.  */

void fft2t(float* x, size_t nx, const float* const w);

/* Radix-2, decimation-in-frequency FFT.  Like fft2r, except that the
   output is in bit-reversed order. */

void fft2f(float* x, size_t nx, const float* const w);

/* Generates basis functions for length nx FFTs */

void initBasis(float* x, size_t nx);

/* Generates basis functions for length nx inverse FFTs */

void initInvBasis(float* x, size_t nx);

/* simple log2 calculation for size_t's.  Dies if input is not a
   power of 2 */

size_t fastLog2(size_t n);

/* returns the smallest power of 2 greater than or equal to n */

size_t nextPowerOf2(size_t n);

/* tests if n is a power of 2 */

int isPowerOf2(size_t n);

#endif  /* fft_h_INCLUDED */
