/*
** $Header: /u/spert/src/fltvec/nv/RCS/scalar.c,v 1.12 1996/04/10 23:04:14 davidj Exp $
**
** Floating point scalar operations - the quick and easy way
*/

#include <fltv-config.h>
#include <stddef.h>
#include <math.h>
#include <fltv-defs.h>
#include <fltv-scansrc.h>

#ifdef DECL_nv_conv_i32_f
/*
** Convert an integer to single precision floating point, rounding if
** necessary.
*/
float nv_conv_i32_f(fltv_int32_t in);
#endif

#ifdef DEFN_nv_conv_i32_f
float
nv_conv_i32_f(fltv_int32_t in)
{
    /* Rely on rounding by default - works on typical 32 bit RISC processors */
    return ((float) in);
}
#endif

/*
** Sigmoid function = 1/(1+e^-x)
*/

#ifdef DECL_nv_sigmoid_f_f
float nv_sigmoid_f_f(float x);
#endif

#ifdef DEFN_nv_sigmoid_f_f
float
nv_sigmoid_f_f(float x)
{
#ifdef FLTV_HAVE_EXPF
    return 1.0f/(1.0f + expf(-x));
#else
    return 1.0f/(1.0f + exp(-x));
#endif
}
#endif /* #ifdef DEFN_nv_sigmoid_f_f */

/*
** Our interface to exponential
*/

#ifdef DECL_nv_exp_f_f
float nv_exp_f_f(float x);
#endif

#ifdef DEFN_nv_exp_f_f
float
nv_exp_f_f(float x)
{
#ifdef FLTV_HAVE_EXPF
    return(expf(x));
#else
    return(exp(x));
#endif
}
#endif

/*
** "error function" - see "man erf" for details
*/

#ifdef DECL_nv_erf_f_f
float nv_erf_f_f(float x);
#endif

#ifdef DEFN_nv_erf_f_f
float
nv_erf_f_f(float x)
{
    return erf(x);
}
#endif /* #ifdef DEFN_nv_erf_f_f */

/*
** Inverse of "error" function - see "man erf" for details
** Calculated using Newtons method
*/

#ifdef DECL_nv_ierf_f_f
float nv_ierf_f_f(float x);
#endif

#ifdef DEFN_nv_ierf_f_f
float
nv_ierf_f_f(float x)
{

     float last = 1.0;		/* Current approximate ierf val */
     float curr = 0.0;		/* Previous approximate ierf val */
     float erfap;		/* Erf() of current approximate ierf */
     float slope;		/* Slope at current point on ierf curve */
     const float DELTA = (1.0 / (1 << 10)); /* Delta use to calc. slope */
     const float LIMIT = (1.0 / (1 << 20)); /* Stop when error this small */
     
     while(fabs(last - curr) > LIMIT) {
	  erfap = erf(curr);
	  slope = (erf(curr + DELTA) - erf(curr)) / DELTA;
	  
	  last = curr;
	  curr += (x - erfap) / slope;
     }
     return(curr);
}
#endif /* #ifdef DEFN_nv_ierf_f_f */

#ifdef DECL_nv_isnan_f_b
/*
** Check whether a signle precision floating point number is a NaN.
** Relies on proper IEEE floating point.
*/
fltv_bool_t nv_isnan_f_b(float f);
#endif

#ifdef DEFN_nv_isnan_f_b
fltv_bool_t
nv_isnan_f_b(float f)
{
    /* With IEEE FP, a NaN does not equal anything */
    return ((f==f) ? FLTV_FALSE : FLTV_TRUE);
}
#endif

#ifdef DECL_nv_isnan_d_b
/*
** Check whether a double precision floating point number is a NaN
** Relies on proper IEEE floating point.
*/
fltv_bool_t nv_isnan_d_b(double d);
#endif

#ifdef DEFN_nv_isnan_d_b
fltv_bool_t
nv_isnan_d_b(double d)
{
    return ((d==d) ? FLTV_FALSE : FLTV_TRUE);
}

#endif

