/*
   $Header: /u/spert/src/fltvec/testsuite/RCS/vecele_vf_vf_test.c,v 1.3 1996/03/22 03:39:01 davidj Exp $

   Test element wise vector routines of the form op_vfvf_vf.
*/

#include <stddef.h>
#include <stdlib.h>

#include "fltvec_test.h"


typedef void (*rtn_t)(size_t len, const float* in,
		      float* res);

/* Performs one test given input values and expected output. */

void
onetest_normal(rtn_t rtn, size_t len,
	const float* in, const float* correct)
{
    /* Create padded copies of inputs. */
    float* c_in = rtst_padvec_clone_vf(len, in);
    float* c_res = rtst_padvec_new_vf(len);

    /* Run routine. */
    rtn(len, c_in, c_res);

    /* Check result. */
    rtst_checknear_fvfvf(len, 1e-6, c_res, correct);

    /* Check input not corrupted. */
    rtst_checkeq_vfvf(len, in, c_in);

    /* Free and check padding. */
    rtst_padvec_del_vf(c_in);
    rtst_padvec_del_vf(c_res);
}

void
neg_5(rtn_t rtn)
{
    const float in[] = { 1.0f, -1.0f, -0.0f, 0.0f, 0.3f };
    const float correct[] = { -1.0f, 1.0f, 0.0f, -0.0f, -0.3f };

    onetest_normal(rtn, 5, in, correct);
}

void
abs_5(rtn_t rtn)
{
    const float in[] = { 1.0f, -1.0f, -0.0f, 0.0f, -0.3f };
    const float correct[] = { 1.0f, 1.0f, 0.0f, 0.0f, 0.3f };

    onetest_normal(rtn, 5, in, correct);
}

/* Compare the results of two vector routines */

void
random_test(rtn_t nv_rtn, rtn_t test_rtn)
{
    const int numtests = rtst_numtests;
    const size_t maxlen = rtst_sizetests;
    float *in = malloc_vf(maxlen);
    float *res = malloc_vf(maxlen);

    int i;
    for (i=0; i<numtests; i++)
    {
        size_t len = (size_t) rtst_urand_i32i32_i32(1, maxlen);

        rtst_urand_ff_vf(len, -1e8, 1e8, in);
        nv_rtn(len, in, res);

        rtst_log("len=%d\n", len);

        onetest_normal(test_rtn, len, in, res);
    }

    free_vf(in);
    free_vf(res);
}

void
test_neg(void)
{
    rtst_start("neg_vf_vf");
    rtst_log("testing nv.\n");
    neg_5(nv_neg_vf_vf);
#ifdef CHECK_NFP
    rtst_log("testing nfp.\n");
    neg_5(nfp_neg_vf_vf);
    random_test(nv_neg_vf_vf, nfp_neg_vf_vf);
#endif
    rtst_passed();
}

void
test_abs(void)
{
    rtst_start("abs_vf_vf");
    rtst_log("testing nv.\n");
    abs_5(nv_abs_vf_vf);
#ifdef CHECK_NFP
    rtst_log("testing nfp.\n");
    abs_5(nfp_abs_vf_vf);
    random_test(nv_abs_vf_vf, nfp_abs_vf_vf);
#endif
    rtst_passed();
}



int
main(int argc, char *argv[])
{
    rtst_args(argc, argv);

    test_neg();
    test_abs();

    rtst_exit();

    return 0;
}
