/*
** $Header: /u/spert/src/fltvec/tor/RCS/conv_gf_gx16s.S,v 1.5 1995/10/13 03:31:58 davidj Exp $
*/

#include "tor/tordef.h"
#include "m_regdef.h"

	.globl tor_conv_gf_gx16s
#ifdef DECL_tor_conv_gf_gx16s 
void tor_conv_gf_gx16s(void); 
#endif

#ifdef DEFN_tor_conv_gf_gx16s

#define vA va0			/* Input A				*/
#define zExp t3			/* Exponent of Z			*/
#define vZ vv2			/* Output Z				*/
#define saturationFlags v0	/* Flags indicating whether each vector	*/
				/*     result element saturated		*/

#define vZExp vt1		/* Exponent of vZ, possibly adjusted	*/
				/*     for subnormal vA			*/
#define zExp2 t4		/* Exponent of Z			*/
#define vNotShiftAmount vt2	/* 31 minus (amount to shift down	*/
				/*     significand of vA minus 4)	*/
#define vAbsA vt0		/* vA shifted up one bit to remove sign	*/
#define vASignif vt1		/* Significand of vA			*/
#define vNegASignif vt2		/* Negative significand of vA		*/
#define vShiftedA va2		/* vAbsA shifted for count of leading	*/
				/*     zeros				*/
#define vCountLeadingZerosA vt1	/* Number of leading zeros of vAbsA	*/
#define vCount16 vt2		/* vCountLeadingZerosA & 0x10		*/
#define vCount8 vt3		/* vCountLeadingZerosA & 0x08		*/
#define vLowIndex vt4		/* Index into low zeros-count table	*/
#define vHighIndex vt5		/* Index into high zeros-count table	*/
#define vAExpOffset vt1		/* Normalization adjustment of exponent	*/
				/*     of vA				*/
#define vAExp vt1		/* Exponent of vA			*/
#define vAIsNaNOrInfinity vt2	/* Flag for vA is a NaN or infinity	*/
#define vAIsNegInfinity vt3	/* Flag for vA is negative infinity	*/

#define temp0 v1
#define temp1 t5
#define temp2 t6
#define config0 t7
#define config1 t8
#define config2 t9

	.ent tor_conv_gf_gx16s

/*
-------------------------------------------------------------------------------
Routine:  tor_conv_gf_gx16s

Input:
	vlr
	va0	vA
	t3	zExp

Output:
	vv2	vZ
	v0	saturationFlags

Not preserved:
	vtt
	vv3
	va0--va3
	vt0--vt5
	v1
	t4--t9

Function:

The single-precision floating-point vector vA is converted to a 16-bit fixed-
point vector and the result returned in vZ.  The exponent of the vZ vector is
given by zExp.  The result is rounded to nearest/even and saturated as
necessary.  The vlr register controls the elements of vZ actually updated.  The
floating-point format conforms to the IEEE Standard for Binary Floating-Point
Arithmetic, and the conversion is performed in a manner consistent with that
standard.  The saturationFlags result indicates for each element in vZ whether
that result saturated.

Exceptions for any of the element-wise conversions are signalled by raising
element-wise flags in the global variable fltflags_invalid32.  A flag in
fltflags_invalid32 is raised if the corresponding element in vA is a NaN.  The
result of attempting to convert a NaN is the maximum positive value 0x7FFF; the
corresponding flag in saturationFlags is _not_ set.

-------------------------------------------------------------------------------
*/

tor_conv_gf_gx16s:

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	va0	vA
|	vt1	vZExp
|	t3	zExp
|	t4	zExp2
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Initially copy zExp to zExp2 and vZExp.  vZExp may later be modified, if vA is
subnormal and if that needs to be taken into account in determining vZ.  In
that case, zExp2 will be set to zero to show that subnormals have already been
dealt with.  (If zExp starts off as zero, subnormals do not need to be taken
into account in determining vZ.)
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	mov.vs vZExp,zExp
	move zExp2,zExp

repeatAfterNormalization:

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	va0	vA
|	vt1	vZExp
|	vt2	vNotShiftAmount
|	t3	zExp
|	t4	zExp2
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Extract the exponent of vA, adjust it, and subtract vZExp from it to form
vNotShiftAmount = 31 - ((the amount to shift down the significand of vA to form
the final result) - 4).  (A shift of 4 bits occurs at the end.  The subtraction
from 31 makes the following saturation cheaper.)  Saturate this to at most 30
and no less than 0, corresponding to a minimum shift of 5 and a maximum of
35 bits.  (It would be possible to use a slightly different range.)
    Subnormal, infinity, and NaN inputs are (mis)treated as though they were
normal.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li config0,((1*FX_SHLAM)|FX_AU|FX_BU\
	            |FX_SEPSHAM|(24*FX_SHRAM)|FX_NOROUND|FX_CLIP_PASS)
	li temp0,((0x7F+12-31)<<24)
	fxsub.vs vNotShiftAmount,vA,temp0,config0
	ssub.vv vNotShiftAmount,vNotShiftAmount,vZExp
	li config1,(FX_LURES|FX_CLIP_PASS|FX_GTZ)
	li temp1,30
	fxsub.vs vNotShiftAmount,vNotShiftAmount,temp1,config1
	cmvltz.vv vNotShiftAmount,vNotShiftAmount,vzero

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	va0	vA
|	vt0	vAbsA
|	vt1	vASignif
|	vt2	vNotShiftAmount
|	t3	zExp
|	t4	zExp2
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Shift up vA by one bit to eliminate its sign.  If the resulting vAbsA is
nonzero, make the leading significant bit explicit by overwriting the bottom
bit of the exponent with a 1.  The exponent of vAbsA may thus be 1 greater than
that vA.
    Subnormal, infinity, and NaN inputs continue to be (mis)treated as though
they were normal.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	addu.vv vAbsA,vA,vA
	li config2,(FX_A_OR_B|FX_SHRV|FX_LURES|FX_CLIP_PASS|FX_NEZ)
	li temp2,(1<<24)
	fxadd.vs vAbsA,vAbsA,temp2,config2

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Extract the significand of vAbsA by shifting up, and then shift down by
31 - vNotShiftAmount, rounding to odd (jamming).  (Recall that vNotShiftAmount
is in the range 0 to 30.)
    Subnormal, infinity, and NaN inputs continue to be (mis)treated as though
they were normal.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li config0,((7*FX_SHLAM)|FX_AU|FX_NOT_B\
	            |FX_SHRL|FX_SHRV|FX_RODD|FX_CLIP_PASS)
	fxadd.vv vASignif,vAbsA,vNotShiftAmount,config0

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	vv0	vZ
|	va0	vA
|	vt0	vAbsA
|	vt1	vASignif
|	vt2	vNegASignif
|	v0	saturationFlags
|	t3	zExp
|	t4	zExp2
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
If vA is negative, negate vASignif.
    Subnormal, infinity, and NaN inputs continue to be (mis)treated as though
they were normal.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	negu.vv vNegASignif,vASignif
	cmvltz.vv vASignif,vA,vNegASignif

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Form the final result by shifting vASignif down the remaining four bits,
rounding to nearest/even and saturating to 16 bits.  Record any saturations in
saturationFlags.
    Subnormal, infinity, and NaN inputs continue to be (mis)treated as though
they were normal.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	ctvu zero,vsat
	li config1,((4*FX_SHLAM)|FX_RNEARESTEVEN|FX_CLIP16)
	fxadd.vv vZ,vzero,vASignif,config1
	cfvu saturationFlags,vsat

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
If vAbsA is nonzero and the exponent of vAbsA is less than 0x80, map vAbsA to
-vAbsA (the negative of vAbsA).  This has the effect in those cases of mapping
the exponent of vAbsA either to (0x100 - exponent) or to (0xFF - exponent).
The mapping cannot overflow; the result of the mapping must be an exponent of
no more than 0xFF.  If the input vA was a subnormal (nonzero with exponent 0),
or if it was an infinity or NaN (exponent 0xFF), the resulting vAbsA must have
an unsigned integer value greater than 0xFE000000.  If that is the case, jump
to a special case handler.  (If vAbsA is zero, -vAbsA is also zero, so the jump
is not taken.  Recall that coming into this code the exponent of vAbsA may have
been 1 more than that of vA.  The case of vAbsA equal to 0xFE000000 corresponds
to an exponent of vA of either 2 or 0xFE.)
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li config2,(FX_CLIP_PASS|FX_LTZ)
	fxsub.vv vAbsA,vzero,vAbsA,config2
	ctvu zero,vcond
	li temp0,0xFE000000
	fltu.sv temp0,vAbsA
	cfvu temp0,vcond
	bnez temp0,specialCases

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Return.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	j ra

specialCases:

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	vv0	vZ
|	va0	vA
|	vt0	vAbsA
|	v0	saturationFlags
|	t3	zExp
|	t4	zExp2
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Shift up vA by one bit to eliminate its sign.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	addu.vv vAbsA,vA,vA

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Determine whether subnormal inputs need attention.  If the code to handle
subnormals has already been executed once, zExp2 is zero; do not execute it
again.  Alternatively, if the fixed-point result exponent is outside the range
-0x95 to -0x6E inclusive, conversion of a subnormal floating-point input will
either saturate or round to zero.  In that case, subnormal inputs can be
treated as though they were normal without danger, so skip the code to handle
subnormals.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	slt temp0,zExp2,-0x6D
	beqz temp0,subnormalsIrrelevant
	slt temp1,zExp2,-0x95
	bnez temp1,subnormalsIrrelevant

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	va0	vA
|	va2	vShiftedA
|	vt0	vAbsA
|	vt1	vCountLeadingZerosA
|	vt2	vCount16
|	vt3	vCount8
|	vt4	vLowIndex
|	vt5	vHighIndex
|	t3	zExp
|	t4	zExp2
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Set vCountLeadingZerosA to the number of leading zeros of vAbsA.
    (NOTE:  This block of code could all be replaced by a single count-leading-
zeros instruction.  See countlz_gi32_gi32.S for more refined commentary.)
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li temp0,(1<<16)
	sltu.vs vCount16,vAbsA,temp0
	li temp1,4
	sllv.vs vCount16,vCount16,temp1
	sllv.vv vShiftedA,vAbsA,vCount16
	li temp2,(1<<24)
	sltu.vs vCount8,vShiftedA,temp2
	li temp0,3
	sllv.vs vCount8,vCount8,temp0
	sllv.vv vShiftedA,vShiftedA,vCount8
	addu.vv vCount8,vCount16,vCount8
	li config0,(FX_SHLZA|FX_A_AND_B|(23*FX_SHLAM)|FX_NOROUND|FX_CLIP_PASS)
	li config1,(FX_SHLV|(28*FX_SHLAM)|FX_NOROUND|FX_CLIP_PASS|FX_GEZ)
	li temp1,(7<<25)
	fxadd.vs vLowIndex,vShiftedA,temp1,config0
	li temp2,0x76554444
	fxadd.sv vCountLeadingZerosA,temp2,vLowIndex,config1
	li temp0,(7<<28)
	addu config2,config0,(3*FX_SHLAM)
	fxadd.vs vHighIndex,vShiftedA,temp0,config2
	li temp1,0xF3221111
	fxadd.sv vCountLeadingZerosA,temp1,vHighIndex,config1
	cmvltz.vv vCountLeadingZerosA,vShiftedA,vzero
	addu.vv vCountLeadingZerosA,vCount8,vCountLeadingZerosA

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	va0	vA
|	vt0	vAbsA
|	vt1	vCountLeadingZerosA,vAExpOffset,vZExp
|	t3	zExp
|	t4	zExp2
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
If vAbsA is subnormal, normalize it by shifting up.  Set vAExpOffset to the
number of bits shifted (zero if normal, infinity, or NaN).
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li temp2,7
	subu.vs vAExpOffset,vCountLeadingZerosA,temp2
	cmvltz.vv vAExpOffset,vAExpOffset,vzero
	sllv.vv vAbsA,vAbsA,vAExpOffset

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Form a normalized vA by combining the normalized vAbsA with the sign from the
old vA.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li temp0,1
	srlv.vs vAbsA,vAbsA,temp0
	li temp1,0x80000000
	and.vs vA,vA,temp1
	or.vv vA,vA,vAbsA

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Combine vAExpOffset with zExp2 to give vZExp, and set zExp2 to zero to signal
that subnormals have already been normalized.  Start over.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	addu.vs vZExp,vAExpOffset,zExp2
	li zExp2,0
	b repeatAfterNormalization

subnormalsIrrelevant:

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
|	vv0	vZ
|	va0	vA
|	vt0	vAbsA
|	vt1	vAExp
|	vt2	vAIsNaNOrInfinity
|	vt3	vAIsNegInfinity
|	v0	saturationFlags
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Extract exponent from vAbsA.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li temp0,24
	srlv.vs vAExp,vAbsA,temp0

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Set vAIsNaNOrInfinity nonzero exactly if vA is a NaN or infinity.  Set
vAIsNegInfinity nonzero exactly if vA is negative infinity.  If vA is a NaN or
is positive infinity, set vZ to 0x7FFF; else if vA is negative infinity, set vZ
to -0x8000.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li temp1,0xFF
	seq.vs vAIsNaNOrInfinity,vAExp,temp1
	li temp2,0x7FFF
	cmvnez.vs vZ,vAIsNaNOrInfinity,temp2
	li temp0,0xFF800000
	seq.vs vAIsNegInfinity,vA,temp0
	li temp1,0x8000
	cmvnez.vs vZ,vAIsNegInfinity,temp1

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
If vA is an infinity, raise the corresponding saturation flag.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	li temp2,0xFF000000
	feq.vs vAbsA,temp2
	cfvu temp0,vcond
	or saturationFlags,temp0

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
If vA is a NaN, raise the corresponding flag in fltflags_invalid32 and clear
the corresponding saturation flag.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	fltu.sv temp2,vAbsA
	cfvu temp1,vcond
	lw temp0,fltflags_invalid32
	or temp0,temp1
	sw temp0,fltflags_invalid32
	not temp1
	and saturationFlags,temp1

/*
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Return.
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
	j ra

	.end tor_conv_gf_gx16s

#endif
