/*
** $Header: /u/spert/src/fltvec/tor/RCS/maxmin.S,v 1.4 1995/12/21 00:39:25 davidj Exp $
**
** Vector single precision floating point maximum and minimum routines
*/

/*
** Note - this file is compiled multiple times to produce several variations
** on a theme.
*/


#include "tor/tordef.h"
#include "tor/fregdef.h"
#include "m_regdef.h"

	.extern	tor_cmp_gfgf_gi32gb
	.extern tor_fltv_bindex, 32


#define INF (0x7f800000)
#define NINF (0xff800000)


#ifdef DECL_tor_imax_vf_u
size_t tor_imax_vf_u(size_t n, const float* v);
#endif

#ifdef DECL_tor_imin_vf_u
size_t tor_imin_vf_u(size_t n, const float* v);
#endif

#ifdef DECL_tor_iabsmax_vf_u
size_t tor_iabsmax_vf_u(size_t n, const float* v);
#endif

#ifdef DECL_tor_iabsmin_vf_u
size_t tor_iabsmin_vf_u(size_t n, const float* v);
#endif

/* Here we define the various preprocessor symbols that differentiate between
   the various max and min routines that are produced from this one source.

TOR_OP_VF_U - The name of the routine
BOTTOM - The "lowest" possible floating point value
CMV_VV - A conditional move that selects the "highest" value
DO_ABS - Defined if we have to take the absolute value of the input
*/

#ifdef DEFN_tor_imax_vf_u
#define TOR_OP_VF_U tor_imax_vf_u
#define BOTTOM NINF
#define CMV_VV cmvltz.vv
#endif

#ifdef DEFN_tor_iabsmax_vf_u
#define TOR_OP_VF_U tor_iabsmax_vf_u
#define BOTTOM NINF
#define CMV_VV cmvltz.vv
#define DO_ABS
#endif

#ifdef DEFN_tor_imin_vf_u
#define TOR_OP_VF_U tor_imin_vf_u
#define BOTTOM INF
#define CMV_VV cmvgtz.vv
#endif

#ifdef DEFN_tor_iabsmin_vf_u
#define TOR_OP_VF_U tor_iabsmin_vf_u
#define BOTTOM INF
#define CMV_VV cmvgtz.vv
#define DO_ABS
#endif


#define length a0		/* The remaining length of the input vector */
#define inPtr a1		/* A pointer to the next bit of input vector */

#define absMask a2		/* A mask used to find the absolute value */
#define seenNan a3		/* Non-zero if we have seen a NaN */

#define savedRA t0		/* The return address from this routine */
#define autoInc t1		/* The auto increment for the vector load */
#define done t2			/* 1 if we have finished the loop */
#define maxvlr t3		/* The maximum vector length (eg 32) */

#define vMaxVal va0		/* The maximum value found so far */
#define vCurVal va1		/* The current value loaded */
#define vIndex va2		/* Indices for the current value loaded */
#define vMaxIndex va3		/* The maximum index we have found so far */
#define vDiff vz0		/* The result - -ve, 0 or +ve if vUnord==0,
				 else undefined */
#define vUnord vz1		/* 1 if unordered */

	.globl	TOR_OP_VF_U

	.ent	TOR_OP_VF_U
TOR_OP_VF_U:

/** Initialization **/

	li	maxvlr, MAXVLEN
        ctvu    maxvlr, vlr
	la	v0, tor_fltv_bindex
	lb.v	vIndex, v0
	li	seenNan, 0
	move	savedRA, ra
#ifdef DO_ABS
	li	absMask, 0x7fffffff
#endif
        bgtu    length, maxvlr, long

/** Handle <= MAXVLEN elements **/

short:
	li	v0, BOTTOM
	mov.vs	vMaxVal, v0
	mov.vv	vMaxIndex, vIndex
	ctvu	length, vlr
	lw.v	vMaxVal, inPtr
#ifdef DO_ABS
	and.vs	vMaxVal, vMaxVal, absMask
#endif
	li	length, MAXVLEN/2
	b	reduce


/** The start of the > MAXVLEN elements loop **/

long:
	li	autoInc, MAXVLEN*4

	lwai.v	vMaxVal, inPtr, autoInc
#ifdef DO_ABS
	and.vs	vMaxVal, vMaxVal, absMask
#endif
	mov.vv	vMaxIndex, vIndex
	subu	length, MAXVLEN
	sltu	done, length, MAXVLEN+1
	lwai.v	vCurVal, inPtr, autoInc
#ifdef DO_ABS
	and.vs	vCurVal, vCurVal, absMask
#endif
	addu.vs	vIndex, vIndex, maxvlr
	bnez	done, loopend

/* The main MAXVLEN-at-a-time reduction */

loop:
	jal	tor_cmp_gfgf_gi32gb
	or	seenNan, v0

	CMV_VV	vMaxVal, vDiff, vCurVal
	CMV_VV	vMaxIndex, vDiff, vIndex
	lwai.v	vCurVal, inPtr, autoInc
#ifdef DO_ABS
	and.vs	vCurVal, vCurVal, absMask
#endif

	addu.vs	vIndex, vIndex, maxvlr

	subu	length, MAXVLEN
	sltu	done, length, MAXVLEN+1
	beqz	done, loop

/* The tail of the main reduction loop */

loopend:
	ctvu	length, vlr
	jal	tor_cmp_gfgf_gi32gb
	or	seenNan, v0
	CMV_VV	vMaxVal, vDiff, vCurVal
	CMV_VV	vMaxIndex, vDiff, vIndex

	li	length, MAXVLEN/2

/* Reduce the 32 maximums to one */

reduce:
	ctvu	length, vlr
redloop:
	vext.v	vCurVal, vMaxVal, length
	vext.v	vIndex, vMaxIndex, length

	jal	tor_cmp_gfgf_gi32gb
	or	seenNan, v0

	CMV_VV	vMaxVal, vDiff, vCurVal
	CMV_VV	vMaxIndex, vDiff, vIndex

	srl	length, 1
	ctvu	length, vlr
	bnez	length, redloop

	vext.s	v0, vMaxIndex, zero

/* Wind up */

finish:
	/* A clever way to retun 0xffffffff if seenNan is not zero */
	.set	noreorder
	bnezl	seenNan, finish2
	li	v0, 0xffffffff
	.set	reorder
finish2:
	j	savedRA
	.end	TOR_OP_VF_U
