/*
** $Header: /u/spert/src/intvec/nv/RCS/svecele.c,v 1.2 1996/10/25 00:40:34 davidj Exp $
**
** Strided element wise vector operations - the quick and easy way.
*/

#include <intv-config.h>
#include <stddef.h>
#include <intv-defs.h>
#include <intv-scansrc.h>

/*******************************************/
/*** One input strided vector/vector ops ***/
/*******************************************/

#ifdef DECL_nv_copy_i8_svi8
/*
** Copy a 8 bit value into a strided vector.
** Simple elementwise implementation.
*/
void nv_copy_i8_svi8(size_t len, size_t stride, intv_int8_t val,
		     intv_int8_t* to);
#endif

#ifdef DEFN_nv_copy_i8_svi8
void
nv_copy_i8_svi8(size_t len, size_t stride, intv_int8_t val,
		  intv_int8_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
    {
	*to = val;
	to += stride;
    }
}
#endif

/****************************************************************/

#ifdef DECL_nv_copy_i16_svi16
/*
** Copy a 16 bit value into a strided vector.
** Simple elementwise implementation.
*/
void nv_copy_i16_svi16(size_t len, size_t stride, intv_int16_t val,
		       intv_int16_t* to);
#endif

#ifdef DEFN_nv_copy_i16_svi16
void
nv_copy_i16_svi16(size_t len, size_t stride, intv_int16_t val,
		  intv_int16_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
    {
	*to = val;
	to += stride;
    }
}
#endif

/****************************************************************/
#ifdef DECL_nv_copy_i32_svi32
/*
** Copy a 32 bit value into a strided vector.
** Simple elementwise implementation.
*/
void nv_copy_i32_svi32(size_t len, size_t stride, intv_int32_t val,
		       intv_int32_t* to);
#endif

#ifdef DEFN_nv_copy_i32_svi32
void
nv_copy_i32_svi32(size_t len, size_t stride, intv_int32_t val,
		  intv_int32_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
    {
	*to = val;
	to += stride;
    }
}
#endif

/****************************************************************/
#ifdef DECL_nv_copy_vi16_svi16
/*
** Copy a 16 bit dense vector into a strided vector.
** Simple elementwise implementation.
*/
void nv_copy_vi16_svi16(size_t len, size_t stride, const intv_int16_t* from,
			intv_int16_t* to);
#endif

#ifdef DEFN_nv_copy_vi16_svi16
void
nv_copy_vi16_svi16(size_t len, size_t stride, const intv_int16_t* from,
		   intv_int16_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
    {
	*to = *from;
	from++;
	to += stride;
    }
}
#endif

/****************************************************************/
#ifdef DECL_nv_copy_svi16_vi16
/*
** Copy a 16 bit strided vector into a dense vector.
** Simple elementwise implementation.
*/
void nv_copy_svi16_vi16(size_t len, size_t stride, const intv_int16_t* from,
			intv_int16_t* to);
#endif

#ifdef DEFN_nv_copy_svi16_vi16
void
nv_copy_svi16_vi16(size_t len, size_t stride, const intv_int16_t* from,
		   intv_int16_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
    {
	*to = *from;
	from += stride;
	to += 1;
    }
}
#endif

