// -*- C++ -*-
// 
// Author: C. Wooters
// Date: 09/06/01
//
#ifndef __LINALG__
#define __LINALG__

#include <valarray>

using namespace std;

class linalg {
 public:
  // some useful constants
  static const double tiny = 1.0e-20;
  static const double epsilon = 0.0000001;

  // the static methods
  static void ludcmp(valarray<double>&, const int, valarray<int>&, float*);

  static void lubksb(const valarray<double>&, const int, 
		     const valarray<int>&, valarray<double>&);

  static void linearSolve(const valarray<double>& a, const double eigVal, 
		    const int n, valarray<double>& y, 
		    const valarray<double>& x);

  static void getEigenVals(valarray<double>& a, const int n, 
			   valarray<double>& wr);

  static void getEigenVec(const valarray<double>& a, double& eigVal, 
			  valarray<double>& eigVec, const int n);

  static void eigenanalysis(const valarray<double>& a, const int n,
			    valarray<double>& e_vals, 
			    valarray<double>& e_vecs);

  static void eigsrta(valarray<double>& d,valarray<double>& v,const int n);

  static void balance(valarray<double>& a, const int n);

  static void elmhes(valarray<double>& a, const int n);

  static void hqr(valarray<double>& a, const int n, 
		  valarray<double>& wr, valarray<double>& wi);

  static void invAxB(valarray<double>&, valarray<double>&,
		     const valarray<double>&, const int);

  static void matinv(valarray<double>&, valarray<double>&, const int);

  static void outerProd(valarray<double>&, const valarray<double>&,
			const valarray<double>&);

  static double innerProd(const valarray<double>& x, 
			  const valarray<double>& y);

  static double vecnorm(const valarray<double>&);

  static void normalize(valarray<double>&);

  static void random_unit_vector(valarray<double>&);

  static double euclidian_dist(const valarray<double>& x, 
			       const valarray<double>& y);

  static inline int imax(const int a, const int b) {return (a > b) ? a : b;}
  static inline double sign(const double a, const double b) {
    return (b>=0.0) ? fabs(a) : -fabs(a);
  }

};

#endif
