// $Header: /u/drspeech/src/quicknet/RCS/QN_MLP_MultiSPERTFx3.h,v 1.2 2001/05/14 21:58:47 dpwe Exp $

#ifndef QN_MLP_MultiSPERTFx3_H_INCLUDED
#define QN_MLP_MultiSPERTFx3_H_INLCUDED

/* Must include the config.h file first */
#include <QN_config.h>
#include <stdio.h>
#include "QN_MLP.h"
#include "QN_Logger.h"

/* We can only do MultiSPERT nets if we have the SPC library. */
#if defined(QN_HAVE_LIBSPCMASTER) && defined(QN_HAVE_LIBFXLIB)
#include "spcmaster.h"
#ifdef __cplusplus
extern "C" {
#endif
#define FX_INLINING
#include "qnfxlib.h"
#ifdef __cplusplus
}
#endif

class QN_MLP_MultiSPERTFx3 : public QN_MLP
{
public:
    QN_MLP_MultiSPERTFx3(int a_debug, const char* a_dbgname,
			 size_t a_input, size_t a_hidden,
			 size_t a_output,
			 enum QN_OutputLayerType a_outtype,
			 size_t a_slaves = 1,
			 int a_in2hid_exp = 0,
			 int a_hid2out_exp = 0,
			 size_t a_size_bunch = 12);
    ~QN_MLP_MultiSPERTFx3();


    // Find out the size of the net
    size_t num_layers() const { return 3; };
    size_t num_sections() const { return 4; };
    size_t size_layer(QN_LayerSelector layer) const;
    void size_section(QN_SectionSelector section, size_t* ouput_p,
		      size_t* input_p) const;
    size_t num_connections() const;

    // Find out the bunch size (not part of the abstract interface)
    size_t get_bunchsize();

    // Use and train.
    void forward(size_t n_frames, const float* in, float* out);
    void train(size_t n_frames, const float* in, const float* target,
	       float* out);

    // Access weights.
    void set_weights(enum QN_SectionSelector which,
		     size_t row, size_t col,
		     size_t n_rows, size_t n_cols,
		     const float* weights);
    void get_weights(enum QN_SectionSelector which,
		     size_t row, size_t col,
		     size_t n_rows, size_t n_cols,
		     float* weights) const;

    // Access learning constants.
    void set_learnrate(enum QN_SectionSelector which, float rate);
    float get_learnrate(enum QN_SectionSelector which) const;

private:
    // Forward pass one bunch in pipeline mode
    void forward_bunchP(size_t n_frames, const float* in, float* out);

    // Train one bunch in pipeline mode
    void train_bunchP( size_t n_frames, const float* in,
		      const float* target, float* out);

    enum
    {
	SIZE_METABUNCH = 10000	// The number of frames we pipeline over.
    };
    
    QN_ClassLogger clog;	// Logging object.

    const size_t n_input;	// No of input units
    const size_t n_hidden;	// No of hidden units
    const size_t n_output;	// No of output units

    const enum QN_OutputLayerType out_layer_type; // Type of output layer
						  // (e.g. sigmoid, softmax)

    // These exponenents are user settable
    const int in2hid_exp;
    const int hid2out_exp;

    const size_t n_slaves;       // Number of partial MLPs
    spc_server** slaves;	// n_slaves partial MLPs
    size_t *slaves_hidden;	// n_hidden for each partial net

    const size_t size_bunch;     // input bunch size

    fxint16 *in_y;               // to convert input to fxint16 on host
static    const int in_y_exp = 3;
    float *out_x;                // Sum of partial output layers.

    float *out_dedy;		 // Output error
    float *out_dydx;		 // Output sigmoid difference
    float *out_dedx;             // Feed back error term from output.

    float neg_in2hid_learnrate;	 // Learning rates (local copies)
    float neg_hid2out_learnrate;
    float neg_hid_learnrate;
    float neg_out_learnrate;

    static const char* CLASSNAME; // Name of the class (for errors)
};

#endif /* #if defined(QN_HAVE_LIBSPCMASTER) && defined(QN_HAVE_LIBFXLIB) */


#endif // #define QN_MLP_MultiSPERTFx3_H_INLCUDED

