const char* QN_SentDB_Norm_RCSID = 
   "$Header: /n/lasagna/db/davidj/spert/src/quicknet/H-spert0/../RCS/QN_SentDB_Norm.cc,v 1.3 1996/03/01 03:37:40 davidj Exp $";

/* Must include the config.h file first */
#include <QN_config.h>
#include <stddef.h>
#include "fltvec.h"
#include "QN_types.h"
#include "QN_Logger.h"
#include "QN_SentDB_Norm.h"

// QN_SentDB_Norm is a class used to normalize a QN_SentDB (i.e.
// multiply each of the feature values by a constant specific to that
// feature number)

const char*
QN_SentDB_Norm::CLASSNAME = "QN_SentDB_Norm";


QN_SentDB_Norm::QN_SentDB_Norm(int dbg, QN_SentDB& a_db,
			       const float* a_norm_mean,
			       const float* a_norm_sdevs)
  : debug(dbg),
    db(a_db),
    n_features(db.num_features_in_frame()),
    norm_nmean(malloc_vf(db.num_features_in_frame())),
    norm_invsd(malloc_vf(db.num_features_in_frame()))
{
    size_t i;			// Local counter

    neg_vf_vf(n_features, a_norm_mean, norm_nmean);
    recip_vf_vf(n_features, a_norm_sdevs, norm_invsd);
    if (debug>=1)
    {
	QN_LOG(CLASSNAME, "Normalizing over %i features.", n_features);
    }
    if (debug>=2)
    {
	QN_LOG(CLASSNAME, "Neg means...");
	for (i=0; i<n_features; i++)
	{
	    QN_LOG(CLASSNAME, "  %f", norm_nmean[i]);
	}
	QN_LOG(CLASSNAME, "Inv SDs...");
	for (i=0; i<n_features; i++)
	{
	    QN_LOG(CLASSNAME, "  %f", norm_invsd[i]);
	}
    }
}

QN_SentDB_Norm::~QN_SentDB_Norm()
{
    free_vf(norm_invsd);
    free_vf(norm_nmean);
}

size_t
QN_SentDB_Norm::get_sentences(size_t sentno, size_t num_sents,
			      QNUInt32* sentnos, QNUInt32* framenos,
			      float* ftrs, QNUInt32* labs, float* tgts)
{
    size_t frames;		// Number of frames in sentences
    size_t i;			// Counter

    frames = db.get_sentences(sentno, num_sents, sentnos, framenos,
			      ftrs, labs, tgts);
    assert(frames!=0);
    for (i=0; i<frames; i++)
    {
	add_vfvf_vf(n_features, ftrs, norm_nmean, ftrs);
	mul_vfvf_vf(n_features, ftrs, norm_invsd, ftrs);
	ftrs += n_features;
    }
    return frames;
}

size_t
QN_SentDB_Norm::num_frames()
{
    return db.num_frames();
}

size_t
QN_SentDB_Norm::num_sentences()
{
    return db.num_sentences();
}

size_t
QN_SentDB_Norm::num_frames_in_sents(size_t sentno, size_t num_sents)
{
    return db.num_frames_in_sents(sentno, num_sents);
}

size_t
QN_SentDB_Norm::num_features_in_frame()
{
    return db.num_features_in_frame();
}

size_t
QN_SentDB_Norm::num_labels_in_frame()
{
    return db.num_labels_in_frame();
}

size_t
QN_SentDB_Norm::num_targets_in_frame()
{
    return db.num_targets_in_frame();
}

