const char* QN_SentDB_PFile_rcsid =
    "$Header: /n/lasagna/db/davidj/spert/src/quicknet/RCS/QN_SentDB_PFile.cc,v 1.9 1996/05/24 00:09:36 davidj Exp $";

// The code for the labelled sentence database implemented as a 
// pfile.  Quite a bit of this stuff relies on floats and longs being 32 bits,
// and on files being <2GB long, but all the other pfile stuff would break
// if these were not true anyway.  If you want big files or 64 bit size_ts,
// do not use pfiles!

#include <QN_config.h>
#include <assert.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "fltvec.h"
#include "QN_types.h"
#include "QN_SentDB_PFile.h"
#include "QN_libc.h"
#include "QN_Logger.h"


// The name of this class

const char* QN_SentDB_PFile::CLASSNAME = "QN_SentDB_PFile";


// Constructor.  This opens and analyses the pfile, and builds up some
// information that will be useful for accessing the pfile later.

QN_SentDB_PFile::QN_SentDB_PFile(int dbg, FILE* file,
				 size_t first_ftr,
				 size_t num_ftrs,
				 size_t first_label,
				 size_t num_labels,
				 size_t first_target,
				 size_t num_targets)
  : debug(dbg),
    pfile(dbg, "QN_SentDB_PFile", file, 1),
    n_sents(pfile.num_segs()),
    n_frames(pfile.num_frames()),
    pfile_n_ftrs(pfile.num_ftrs()),
    pfile_n_labs(pfile.num_labs())
{
    assert(first_ftr==0);
    assert(num_ftrs==QN_ALL);
    assert(first_label==0);
    assert(num_labels==QN_ALL);
    assert(first_target==0);
    assert(num_targets==0 || num_targets==QN_ALL);
    assert(num_targets==0);
    // TODO
}

QN_SentDB_PFile::~QN_SentDB_PFile()
{
}

// Given a range of sentences, get the features and labels for those sentences.
// Returns the number of frames read.

size_t
QN_SentDB_PFile::get_sentences(size_t first_sent, size_t num_sents,
			       QNUInt32* sentnos, QNUInt32* framenos,
			       float* ftrs, QNUInt32* labs, float* tgts)
{
    size_t sent;		// Current sentence

    assert((first_sent+num_sents)<=n_sents);

    for (sent=first_sent; sent<first_sent+num_sents; sent++)
    {
	assert(sentnos==NULL);
	assert(framenos==NULL);
	assert(ftrs==NULL);
	assert(tgts==NULL);
	assert(labs==NULL);
    }
    return 0;
}




// Get the number of frames in a given sentence.
size_t
QN_SentDB_PFile::num_frames_in_sents(size_t first_sent,
				     size_t num_sents)
{
    size_t sent;		// Current sentence
    size_t frames = 0;		// Accumulator for number of frames
    
    assert((first_sent+num_sents)<=n_sents);

    for (sent=first_sent; sent<first_sent+num_sents; sent++)
    {
	size_t frames_this_sent; // Frames in current sentence
	
	frames_this_sent = pfile.num_frames(sent);
	if (frames_this_sent!=QN_SIZET_BAD)
	{
	    QN_ERROR(CLASSNAME, "Cannot find number of frames in sentence in "
		     "unindexed PFile\n");
	}
	frames += frames_this_sent;
    }
    return frames;
}
