// $Header: /n/lasagna/db/davidj/spert/src/quicknet/RCS/QN_SentDB_PFile.h,v 1.5 1996/05/24 00:10:24 davidj Exp $

#ifndef QN_SentDB_PFile_h_INCLUDED
#define QN_SentDB_PFile_h_INCLUDED

// QN_SentDB_PFile is an interface to pfiles, the binary data files used
// by the "BoB" connectionist simulator.  Specifically, it works with PFiles
// that contain just features and labels - the majority of "traditional"
// PFiles in the Realization group as of 13th March, 1995.

// ..and also now should handle PFiles containing vectors of float targets.
// Tue Oct  3 12:58:01 1995
// Krste

// BoB allows a user to pick arbitrary features from each frame in the pfile -
// we just allow a contiguous set of features, which will be more efficient
// and works for many cases (e.g. ignoring energy, ignoring deltas).

/* Must include the config.h file first */
#include <QN_config.h>
#include <assert.h>
#include <stdio.h>
#include "QN_types.h"
#include "QN_SentDB.h"
#include "QN_PFile.h"

//*************** WARNING - DOES NOT CURRENTLY WORK ****************
// davidj - 23rd May 1996

class QN_SentDB_PFile : public QN_SentDB
{
public:
    QN_SentDB_PFile(int dbg,
		    FILE* file, size_t first_ftr, size_t num_ftrs,
		    size_t first_label, size_t num_labels,
		    size_t first_target = 0, size_t num_targets = 0);
    ~QN_SentDB_PFile();

    // Return the number of frames in the database.
    size_t num_frames() { return pfile.num_frames(); };
    // Return the number of sentences in the database.
    size_t num_sentences() { return pfile.num_segs(); };
    // Return the number of frames in a range of sentences.
    size_t num_frames_in_sents(size_t first_sent, size_t num_sents);
    // Return the number of features in a frame.
    size_t num_features_in_frame() { return pfile.num_ftrs(); };
    // Return the number of labels in a frame.
    size_t num_labels_in_frame() { return pfile.num_labs(); };
    // Return the number of targets in a frame.
    size_t num_targets_in_frame() { return n_targets; };
    
    // Gets the labels and features for a range of sentences.
    // If sentnos is NULL, do not get sentence numbers.
    // If framenos is NULL, do not get frame numbers.
    // If ftrs is NULL, do not get features.
    // If labs is NULL, do not get labels.
    // If tgts is NULL, do not get targets.
    // Use num_frames_in_sents() first to check space
    size_t get_sentences(size_t first_sent, size_t num_sent,
			 QNUInt32* sentnos, QNUInt32* framenos,
			 float* ftrs, QNUInt32* labs, float* tgts);

    // Some details about the PFile itself - not in abstract class

    // Return the number of feature in a frame in the pfile
    size_t num_features_in_pfile() { return pfile_n_ftrs; };
    // Return the number of labels in a frame in the pfile
    size_t num_labels_in_pfile() { return pfile_n_labs; };
    // Return the number of targets in a frame in the pfile
    size_t num_targets_in_pfile() { return 0; };
    
private:
    const int debug;		// Debug level.

    QN_InFtrLabStream_PFile pfile; // The PFile reader object

    size_t ftr_col;		// Column of first feature we want.
    size_t n_ftrs;		// Number of features we want.
    size_t lab_col;		// Column of first label we want.
    size_t n_labs;		// Number of labels we want.
    size_t target_col;		// Column of first target we want.
    size_t n_targets;		// Number of targets we want.

    const size_t n_sents;	// Number of sentences in PFile
    const size_t n_frames;	// Total number of frames in PFile
    const size_t pfile_n_ftrs;	// Number of featuers in PFile
    const size_t pfile_n_labs;	// Number of labels in PFile
    static const char* CLASSNAME;	// Name of the class (for error
					// reporting) 
};


#endif // #ifdef QN_SentDB_PFile_h_INCLUDED
