// $Header: /n/lasagna/db/davidj/spert/src/quicknet/H-spert0/../RCS/QN_SentDB_Subset.h,v 1.3 1996/03/01 03:37:40 davidj Exp $

#ifndef QN_SentDB_Subset_h_INCLUDED
#define QN_SentDB_Subset_h_INCLUDED

/* Must include the config.h file first */
#include <QN_config.h>
#include <assert.h>
#include <stddef.h>
#include <stdio.h>
#include "QN_types.h"
#include "QN_SentDB.h"

// QN_SentDB_Subset is a class used to access a contiguous subset of
// sentences in a labelled sentence database.

class QN_SentDB_Subset : public QN_SentDB
{
public:
    // Constructor.
    // a_db - the database we are subsetting
    // first - the number of the first sentence we wish to consider
    // num - number of sentences we are using - specify more than required
    //       if all sentences in database needed
    QN_SentDB_Subset(int dbg, QN_SentDB& a_db, size_t first, size_t num);
    ~QN_SentDB_Subset() {};

    // Return the total number of frames in the database
    size_t num_frames();
    // Return the total number of sentences in the database
    size_t num_sentences();
    // Return the number of frames in a range of sentences
    size_t num_frames_in_sents(size_t first_sent, size_t num_sents);
    // Return the number of reatures in a frame
    size_t num_features_in_frame();
    // Return the number of labels in a frame
    size_t num_labels_in_frame();
    // Return the number of targets in a frame
    size_t num_targets_in_frame();

    // Return the labels and features for a given sentence.
    // If ftrs is NULL, do not get features.
    // If labs is NULL, do not get labels.
    size_t get_sentences(size_t sentno, size_t num_sents,
			 QNUInt32* sentnos, QNUInt32* framenos,
			 float* ftrs, QNUInt32* labs, float* tgts);
private:
    int debug;			// Level of verbosity
    QN_SentDB& db;		// The database we are subsetting
    size_t first_sent;		// The first sentence we use
    size_t n_frames;		// The total number of frames in our subset
    size_t n_sentences;		// The number of sentences we supply
    static const char* CLASSNAME; // Name of the class (for error
				// reporting) 
};

inline size_t
QN_SentDB_Subset::num_features_in_frame()
{
    return db.num_features_in_frame();
}

inline size_t
QN_SentDB_Subset::num_labels_in_frame()
{
    return db.num_labels_in_frame();
}

inline size_t
QN_SentDB_Subset::num_targets_in_frame()
{
    return db.num_targets_in_frame();
}

inline size_t
QN_SentDB_Subset::num_frames_in_sents(size_t sentno, size_t num_sents)
{
    assert(sentno < n_sentences && (sentno+num_sents-1 < n_sentences));

    return db.num_frames_in_sents(sentno + first_sent, num_sents);
}

inline size_t
QN_SentDB_Subset::num_frames()
{
    return n_frames;
}

// Return the total number of sentences in the database

inline size_t
QN_SentDB_Subset::num_sentences()
{
    return n_sentences;
}

#endif // #ifdef QN_SentDB_Subset_h_INCLUDED
