. $Header: /n/lasagna/db/davidj/spert/src/quicknet/RCS/qntrain.man,v 1.29 1998/06/12 01:25:47 davidj Exp $
.TH qntrain 1 "$Date: 1998/06/12 01:25:47 $" ICSI "ICSI SPEECH SOFTWARE"
.SH NAME
qntrain \- simple MLP training program using the QuickNet library
.SH SYNOPSIS
.B qntrain
[
.I options
]
.SH DESCRIPTION
.I qntrain
is a simple program for training MLPs (Multi Layer Perceptrons - a.k.a.
Neural Nets).    It implements a subset of the functionality of
\fBqnstrn\fR and is only maintained for backwards compatability.  All
new recognition systems should use \fBqnstrn\fR.
.SH OPTIONS
.I qntrain
can take the following options:
.TP 1i			\" Indent the paragraph
.BI train_ftrfile= filename
Specify the file containing features used to train the net.
.TP
.BI train_ftrfile_format= format
Specify the format of the feature file.  This can be either
.B pfile
(the ICSI feature file format - see
.BR pfile (5))
or
.B pre
- the CUED feature file format.  The default is
.BR pfile .
.TP
.BI train_ftrfile_num_ftrs= integer
Specify the number of features in the training feature file.  The default \- 
.B 0
\- means use the number of features specified in the feature file header.
A non-zero value must be given for feature files that are in a format
that does not include a header, e.g. Cambridge \fBpre\fR files.
.TP
.BI input_normfile= filename
Specify a file containing normalization parameters for both the
training and cross validation feature streams.  The default value \-
an empty string \- means apply no normalization.  In general, training
without normalization is not recommended.
.TP
.BI first_train_sent= integer
The number of the first sentence in the feature file to use for
training
.RB ( 0
means use the first sentence in the feature file).
.TP
.BI num_train_sents= integer
The total number of sentences in the feature file to use for training.
.TP
.BI train_cache_sents= integer
This option exists for historical compatibility only \- the supplied
value is ignored.
.TP
.BI train_cache_frames= integer
The number of frames of training data loaded into the cache used for
presentation randomization.  A variable number of sentences is read
sequentially from the training feature file until this cache is as
close to being full as possible.  This cache is then used to provide
random presentations for training until all frames have been used
once, at which point the next sequence of sentences is loaded.  The
number of frames cached has a significant affect on memory usage, with
one frame requiring approximately (num_ftrs+num_labels+1)*4 bytes.
.TP
.BI first_cv_sent= integer
The number of the first sentence in the feature file to use for
cross validation
.RB ( 0
means use the first sentence in the feature file).
.TP
.BI num_cv_sents= integer
The total number of sentences in the feature file to use for cross validation.
.TP
.BI first_ftr= integer
The first feature to use from the feature file.  This is often
.BR 1 ,
which ignores the energy feature in a typical PLP feature file.
.TP
.BI num_ftrs= integer
The number of features to use from the feature file.  The default \- \fB0\fR
\- means use all features in the feature file after the first feature
specified by the \fBfirst_ftr\fR option.
.TP
.BI input_weightfile= filename
Specifiy a file containing weights to load into the net before
training.  By default, the format of this file is an ASCII weight file
as used by BoB and clones.  Specifying a filename of
.B \-
means use random initial weights.
.TP
.BI output_weightfile= filename
Specify a file in which to save the weights from the trained net.
.TP
.BI log_weightfile= filename
Specify a file in which a copy of the weights are saved at the end of
each epoch.  This file is used when the cross validation error rate for an
epoch is greater than the previous one, in which case the latest weight
updates are abandoned by reloading the weights from this file.

Three special substrings can be used to in the log weightfile
filename. The string
.B %p
is replaced by the process number of the qntrain process.  The string
.B %e
is replaced by the number of the current epoch (allowing weight
logs from all epochs to be retained).  The string
.B %%
is replaced by a single
.BR % "."
.TP
.BI min_random_bias= float
.TP
.BI max_random_bias= float
.TP
.BI min_random_weight= float
.TP
.BI max_random_weight= float
Limits for the random initialization of weights and biases \- see
.BR input_weightfile .
.TP
.BI initial_learnrate= float
This option exists for historical compatibility only \- the supplied
value is ignored.
.TP
.BI learnrate_schedule= schedule
The name of the strategy used for updating the learning rate in
successive training epochs.

\fInewbob\fR means use the same approach as taken by BoB: use a
constant learning rate until the error reduction drops below a given
threshold, then decrease exponentially.  The initial learning rate is
set the first value specified in the \fBlearnrate\fR option.

\fIlist\fR means use the learning rate list specified in the
\fBlearnrate\fR option.  After all learning rates have been used, the
training terminates.

The default value is \fInewbob\fB.
.TP
.BI learnrate= float,float,...
Specify the learning rate for one or more epochs.  See the
\fBlearnrate_schedule\fR option for more details.
.TP
.BI learnrate_scaleby= float
Each epoch, the learning rate is scaled by this factor.  Currently
only used by the \fBnewbob\fR learning rate schedule.
.TP
.BI learnrate_ramp_thresh= float
When the difference between error rates for subsequent epochs drops
below this value, the learning rate starts to ramp down by
\fBlearnrate_scaleby\fR for subsequent epochs.  Currently
only used by the \fBnewbob\fR learning rate schedule.
.TP
.BI learnrate_stop_thresh= float
When the learning rate is ramping and the
difference between error rates for subsequent epochs drops
below this value, the training terminates.
Currently only used by the \fBnewbob\fR learning rate schedule.
.TP
.BI max_epochs= integer
Maximum number of epochs of training.  We stop when we have trained
this many epochs regardless of the learning rate schedule.  if
\fBmax_epochs\fR is 0, we do not train at all.
.TP
.BI window_width= integer
The number of frames of input features to combine to form the input layer
of the MLP.
.TP
.BI window_offset= integer
The offset of the start of the MLP input window from the current frame
being trained.  For example, with a
.B window_width
of
.B 9
, a
.B window_offset
of
.B -4
will result in a symmetric input window.
.TP
.BI num_input= integer
Number of input units in the MLP.
.TP
.BI num_hidden= integer
Number of hidden units in the MLP.
.TP
.BI num_output= integer
Number of output units in the MLP.
.TP
.BI mlp_output_type= unittype
Specify the type of non-linearity to use for the MLP output layer.
Allowable values are \fBsigmoid\fR (sigmoid with least mean squares
error criteria used for training), \fBsigmoidx\fR (sigmoid with relative
entropy error criteria used during training) and \fBsoftmax\fR (again
with relative entropy error criteria).  The default is \fBsoftmax\fR.
.TP
.BI use_fx= bool
Use a fixed point MLP if
.BR true .
.TP
.BI seed= integer
Set the seed for random number generation.  Controls both the random
initialization of weights and random training pattern selection.
.TP
.BI verbose= bool
Output more status messages if
.BR true .
.TP
.BI debug= integer
Set the level of internal debugging output.  \fB0\fR means none,
\fB6\fR means lots!

.SH ENVIRONMENT
Describe the environment variables used here.
.TP 20
.B TZ
Time Zone.  On some systems, this is used for displaying times during
the training run \- if times seem to be wrong by several hours, it is
because this environment variable is not set.

. .SH FILES

.SH AUTHOR
David Johnson  <davidj@ICSI.Berkeley.EDU>
.SH SEE ALSO
.BR qnstrn (1) ,
.BR qnforward (1) ,
.BR qnnorm (1) ,
.BR norms (5) ,
.BR pfile (5) .
.SH NOTES
Many more options need to be added to this program.
.SH BUGS
Older versions of the SPERT libraries and servers do not support TZ \-
if all times are in GMT, upgrade to the latest SPERT development
environment and recompile Quicknet.




